@@ -18,6 +18,7 @@
  *
  * Authors:
  *   Chema Celorio <chema@celorio.com>
+ *   Tristan Van Berkom <tvb@gnome.org>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -87,8 +88,6 @@ glade_property_dup_impl (GladeProperty *template, GladeWidget *widget)
 
 	g_value_init (property->value, template->value->g_type);
 	g_value_copy (template->value, property->value);
-
-	property->i18n_translatable = template->i18n_translatable;
 	
 	return property;
 }
@@ -103,25 +102,40 @@ glade_property_reset_impl (GladeProperty *property)
 gboolean
 glade_property_default_impl (GladeProperty *property)
 {
-	return !g_param_values_cmp (property->class->pspec,
-				    property->value,
-				    property->class->def);
+	return GLADE_PROPERTY_GET_KLASS (property)->equals_value
+		(property, property->class->def);
 }
 
 gboolean
 glade_property_equals_value_impl (GladeProperty *property,
 				  const GValue  *value)
 {
+	if (G_IS_PARAM_SPEC_STRING (property->class->pspec))
+	{
+		const gchar *prop_str, *value_str;
+
+		/* in string specs; NULL and '\0' are 
+		 * treated as equivalent.
+		 */
+		prop_str = g_value_get_string (property->value);
+		value_str = g_value_get_string (value);
+
+		if (prop_str == NULL && value_str && value_str[0] == '\0')
+			return TRUE;
+		else if (value_str == NULL && prop_str && prop_str[0] == '\0')
+			return TRUE;
+	}
+
 	return !g_param_values_cmp (property->class->pspec,
 				    property->value, value);
 }
 
-/**
+/*
  * Generic set function for properties that do not have a
  * custom set_property method. This includes packing properties.
  */
 static void
-glade_property_set_property (GladeProperty *property, const GValue *value)
+glade_property_sync_property (GladeProperty *property, const GValue *value)
 {
 	if (property->class->packing)
 	{
@@ -138,11 +152,69 @@ glade_property_set_property (GladeProperty *property, const GValue *value)
 	}
 }
 
+static void
+glade_property_update_prop_refs (GladeProperty *property, 
+				 const GValue  *old_value,
+				 const GValue  *new_value)
+{
+	GladeWidget *gold, *gnew;
+	GObject     *old_object, *new_object;
+	GList       *old_list, *new_list, *list, *removed, *added;
+
+	if (GLADE_IS_PARAM_SPEC_OBJECTS (property->class->pspec))
+	{
+		/* Make our own copies incase we're walking an
+		 * unstable list
+		 */
+		old_list = g_value_dup_boxed (old_value);
+		new_list = g_value_dup_boxed (new_value);
+
+		/* Diff up the GList */
+		removed = glade_util_removed_from_list (old_list, new_list);
+		added   = glade_util_added_in_list     (old_list, new_list);
+
+		/* Adjust the appropriate prop refs */
+		for (list = removed; list; list = list->next)
+		{
+			old_object = list->data;
+			gold = glade_widget_get_from_gobject (old_object);
+			glade_widget_remove_prop_ref (gold, property);
+		}
+		for (list = added; list; list = list->next)
+		{
+			new_object = list->data;
+			gnew = glade_widget_get_from_gobject (new_object);
+			glade_widget_add_prop_ref (gnew, property);
+		}
+
+		g_list_free (removed);
+		g_list_free (added);
+		g_list_free (old_list);
+		g_list_free (new_list);
+	}
+	else
+	{
+		if ((old_object = g_value_get_object (old_value)) != NULL)
+		{
+			gold = glade_widget_get_from_gobject (old_object);
+			glade_widget_remove_prop_ref (gold, property);
+		}
+		
+		if ((new_object = g_value_get_object (new_value)) != NULL)
+		{
+			gnew = glade_widget_get_from_gobject (new_object);
+			glade_widget_add_prop_ref (gnew, property);
+		}
+	}
+}
+
 static void
 glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 {
-	GladeProject *project;
+	GladeProject *project = property->widget ?
+		glade_widget_get_project (property->widget) : NULL;
 	gboolean      changed = FALSE;
+	GValue old_value = {0,};
 
 	if (!g_value_type_compatible (G_VALUE_TYPE (property->value), G_VALUE_TYPE (value)))
 	{
@@ -151,7 +223,10 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 		return;
 	}
 
-	if (property->class->verify_function)
+	if (glade_property_superuser () == FALSE &&
+	    property->widget &&
+	    property->class->verify_function &&
+	    project && glade_project_is_loading (project) == FALSE)
 	{
 		GObject *object = glade_widget_get_object (property->widget);
 		if (property->class->verify_function (object, value) == FALSE)
@@ -165,25 +240,16 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 
 
 	/* Add/Remove references from widget ref stacks here
+	 * (before assigning the value)
 	 */
-	if (changed && glade_property_class_is_object (property->class))
-	{
-		GladeWidget *gold, *gnew;
-		GObject     *old_object, *new_object;
+	if (property->widget && changed && glade_property_class_is_object (property->class))
+		glade_property_update_prop_refs (property, property->value, value);
 
-		if ((old_object = g_value_get_object (property->value)) != NULL)
-		{
-			gold = glade_widget_get_from_gobject (old_object);
-			glade_widget_remove_prop_ref (gold, property);
-		}
-
-		if ((new_object = g_value_get_object (value)) != NULL)
-		{
-			gnew = glade_widget_get_from_gobject (new_object);
-			glade_widget_add_prop_ref (gnew, property);
-		}
-	}
 
+	/* Make a copy of the old value */
+	g_value_init (&old_value, G_VALUE_TYPE (property->value));
+	g_value_copy (property->value, &old_value);
+	
 	/* Assign property first so that; if the object need be
 	 * rebuilt, it will reflect the new value
 	 */
@@ -192,15 +258,14 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 
 	GLADE_PROPERTY_GET_KLASS (property)->sync (property);
 
-	if (changed)
+	if (changed && property->widget)
 	{
-		if ((project = glade_widget_get_project (property->widget)) != NULL)
-			glade_project_changed (project);
-
 		g_signal_emit (G_OBJECT (property),
 			       glade_property_signals[VALUE_CHANGED],
-			       0, property->value);
+			       0, &old_value, property->value);
 	}
+	
+	g_value_unset (&old_value);
 }
 
 static void
@@ -222,9 +287,18 @@ glade_property_get_default_impl (GladeProperty *property, GValue *value)
 static void
 glade_property_sync_impl (GladeProperty *property)
 {
-	if (property->enabled == FALSE || 
-	    property->class->ignore    ||
-	    property->syncing)
+
+	/* Heh, here are the many reasons not to
+	 * sync a property ;-)
+	 */
+	if (/* optional properties that are disabled */
+	    property->enabled == FALSE    || 
+	    /* explicit "never sync" flag */
+	    property->class->ignore       || 
+	    /* avoid recursion */
+	    property->syncing             ||
+	    /* No widget owns this property yet */
+	    property->widget == NULL)
 		return;
 
 	property->syncing = TRUE;
@@ -243,10 +317,9 @@ glade_property_sync_impl (GladeProperty *property)
 		gboolean  reselect  = FALSE;
 		gboolean  inproject =
 			property->widget->project ?
-			(glade_project_get_widget_by_name
+			(glade_project_has_object
 			 (property->widget->project,
-			  property->widget->name) ? TRUE : FALSE) : FALSE;
-
+			  glade_widget_get_object (property->widget)) ? TRUE : FALSE) : FALSE;
 		
 		if (inproject)
 		{
@@ -278,83 +351,136 @@ glade_property_sync_impl (GladeProperty *property)
 		}
 	}
 	else
-		glade_property_set_property (property, property->value);
+		glade_property_sync_property (property, property->value);
 
 	property->syncing = FALSE;
 }
 
+static void
+glade_property_load_impl (GladeProperty *property)
+{
+	GObject      *object;
+	GObjectClass *oclass;
+	
+	if (property->widget == NULL ||
+	    property->class->packing ||
+	    property->class->type != GPC_NORMAL)
+		return;
+	object = glade_widget_get_object (property->widget);
+	oclass = G_OBJECT_GET_CLASS (object);
+	
+	if (g_object_class_find_property (oclass, property->class->id))
+		g_object_get_property (object, property->class->id, property->value);
+}
+
 static gboolean
 glade_property_write_impl (GladeProperty  *property, 
 			   GladeInterface *interface,
 			   GArray         *props)
 {
-	GladePropInfo info = { 0 };
-	gchar *tmp;
-	gchar *default_str = NULL;
-	gboolean skip;
+	GladePropInfo         info  = { 0, };
+	GladeAtkActionInfo    ainfo = { 0, };
+	GList                *list;
+	gchar                *name, *value, **split, *tmp;
+	gint                  i;
 
 	if (!property->class->save || !property->enabled)
 		return FALSE;
 
+	g_assert (property->class->orig_def);
+	g_assert (property->class->def);
+
+	/* Skip properties that are default
+	 * (by original pspec default) 
+	 */
+	if (glade_property_equals_value (property, property->class->orig_def))
+		return FALSE;
+
 	/* we should change each '-' by '_' on the name of the property 
          * (<property name="...">) */
-	tmp = g_strdup (property->class->id);
-	if (!tmp)
-		return FALSE;
-	glade_util_replace (tmp, '-', '_');
-
-	/* put the name="..." part on the <property ...> tag */
-	info.name = alloc_propname(interface, tmp);
-	g_free (tmp);
-
-	/* convert the value of this property to a string, and put it between
-	 * the opening and the closing of the property tag */
-	tmp = glade_property_class_make_string_from_gvalue (property->class,
-							    property->value);
-	if (!tmp || !tmp[0])
-		return FALSE;
-
-	if (property->class->orig_def == NULL)
+	if (property->class->type != GPC_NORMAL)
 	{
-		/* Skip custom properties that are NULL string types. */
-		skip = G_IS_PARAM_SPEC_STRING (property->class->pspec) &&
-			!g_value_get_string (property->value);
+
+		tmp = (gchar *)glade_property_class_atk_realname (property->class->id);
+		name = g_strdup (tmp);
 	}
 	else
 	{
-		/* Skip properties that are default
-		 * (by original pspec default) 
-		 */
-		default_str =
-			glade_property_class_make_string_from_gvalue
-			(property->class, property->class->orig_def);
-		skip = default_str && !strcmp (tmp, default_str);
-		g_free (default_str);
-	}
-	
-	if (skip)
-	{
-		g_free (tmp);
-		return FALSE;
+		name = g_strdup (property->class->id);
 	}
 
-	if (property->class->translatable)
+	/* convert the value of this property to a string */
+	if (property->class->type == GPC_ACCEL_PROPERTY ||
+	    (value = glade_property_class_make_string_from_gvalue 
+	     (property->class, property->value)) == NULL)
+		/* make sure we keep the empty string, also... upcomming
+		 * funcs that may not like NULL.
+		 */
+		value = g_strdup ("");
+
+	switch (property->class->type)
 	{
-		info.translatable = property->i18n_translatable;
-		info.has_context  = property->i18n_has_context;
-		if (property->i18n_comment)
-			info.comment = alloc_string
-				(interface, property->i18n_comment);
+	case GPC_ATK_PROPERTY:
+		tmp = g_strdup_printf ("AtkObject::%s", name);
+		g_free (name);
+		name = tmp;
+		/* Dont break here ... */
+	case GPC_NORMAL:
+		info.name = glade_xml_alloc_propname(interface, name);
+		info.value = glade_xml_alloc_string(interface,  value);
+		
+		if (property->class->translatable)
+		{
+			info.translatable = property->i18n_translatable;
+			info.has_context  = property->i18n_has_context;
+			if (property->i18n_comment)
+				info.comment = glade_xml_alloc_string
+					(interface, property->i18n_comment);
+		}
+		g_array_append_val (props, info);
+		break;
+	case GPC_ATK_RELATION:
+		if ((split = g_strsplit (value, GPC_OBJECT_DELIMITER, 0)) != NULL)
+		{
+			for (i = 0; split[i] != NULL; i++)
+			{
+				GladeAtkRelationInfo rinfo = { 0, };
+				rinfo.type   = glade_xml_alloc_string(interface, name);
+				rinfo.target = glade_xml_alloc_string(interface, split[i]);
+				g_array_append_val (props, rinfo);
+			}
+			g_strfreev (split);
+		}
+		break;
+	case GPC_ATK_ACTION:
+		ainfo.action_name = glade_xml_alloc_string(interface, name);
+		ainfo.description = glade_xml_alloc_string(interface, value);
+		g_array_append_val (props, ainfo);
+		break;
+	case GPC_ACCEL_PROPERTY:
+		for (list = g_value_get_boxed (property->value); 
+		     list; list = list->next)
+		{
+			GladeAccelInfo *accel = list->data;
+			GladeAccelInfo  accel_info = { 0, };
+
+			accel_info.signal    = glade_xml_alloc_string(interface, accel->signal);
+			accel_info.key       = accel->key;
+			accel_info.modifiers = accel->modifiers;
+
+			g_array_append_val (props, accel_info);
+		}
+		break;
+	default:
+		break;
 	}
 	
-	info.value = alloc_string(interface, tmp);
-	g_array_append_val (props, info);
-	g_free (tmp);
+	g_free (name);
+	g_free (value);
 
 	return TRUE;
 }
 
-
 static G_CONST_RETURN gchar *
 glade_property_get_tooltip_impl (GladeProperty *property)
 {
@@ -480,6 +606,7 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 	prop_class->get_value             = glade_property_get_value_impl;
 	prop_class->get_default           = glade_property_get_default_impl;
 	prop_class->sync                  = glade_property_sync_impl;
+	prop_class->load                  = glade_property_load_impl;
 	prop_class->write                 = glade_property_write_impl;
 	prop_class->get_tooltip           = glade_property_get_tooltip_impl;
 	prop_class->value_changed         = NULL;
@@ -504,7 +631,7 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 		(object_class, PROP_I18N_COMMENT,
 		 g_param_spec_string 
 		 ("i18n-comment", _("Comment"), 
-		  _("XXX FIXME: The translators comment ?"),
+		  _("Comment for translators"),
 		  NULL, G_PARAM_READWRITE));
 
 	g_object_class_install_property 
@@ -518,10 +645,10 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 		(object_class, PROP_I18N_HAS_CONTEXT,
 		 g_param_spec_boolean 
 		 ("i18n-has-context", _("Has Context"), 
-		  _("Whether this property is translatable or not"),
-		  TRUE, G_PARAM_READWRITE));
+		  _("Whether or not the translatable string has a context prefix"),
+		  FALSE, G_PARAM_READWRITE));
 
-	/* Signals */
+	/* Signal */
 	glade_property_signals[VALUE_CHANGED] =
 		g_signal_new ("value-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -529,8 +656,8 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 			      G_STRUCT_OFFSET (GladePropertyKlass,
 					       value_changed),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+			      glade_marshal_VOID__POINTER_POINTER,
+			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
 
 	glade_property_signals[TOOLTIP_CHANGED] =
 		g_signal_new ("tooltip-changed",
@@ -573,26 +700,340 @@ glade_property_get_type (void)
 	return property_type;
 }
 
+/*******************************************************************************
+                        GladeInterface Parsing code
+ *******************************************************************************/
+static GValue *
+glade_property_read_packing (GladeProperty      *property, 
+			     GladePropertyClass *pclass,
+			     GladeProject       *project,
+			     GladeChildInfo     *info,
+			     gboolean            free_value)
+{
+	GValue    *gvalue = NULL;
+	gint       i;
+	gchar     *id;
+
+	for (i = 0; i < info->n_properties; ++i)
+	{
+		GladePropInfo *pinfo = info->properties + i;
+		
+		id = glade_util_read_prop_name (pinfo->name);
+		
+		if (!strcmp (id, pclass->id))
+		{
+			gvalue = glade_property_class_make_gvalue_from_string
+				(pclass, pinfo->value, project);
+			
+			if (property)
+			{
+				glade_property_i18n_set_translatable
+					(property, pinfo->translatable);
+				glade_property_i18n_set_has_context
+					(property, pinfo->has_context);
+				glade_property_i18n_set_comment
+					(property, pinfo->comment);
+
+				property->enabled = TRUE;
+				
+				GLADE_PROPERTY_GET_KLASS (property)->set_value
+					(property, gvalue);
+			}
+
+			if (free_value)
+			{
+				g_value_unset (gvalue);
+				g_free (gvalue);
+			}
+
+			g_free (id);
+			break;
+		}
+		g_free (id);
+	}
+	return gvalue;
+}
+
+static GValue *
+glade_property_read_normal (GladeProperty      *property, 
+			    GladePropertyClass *pclass,
+			    GladeProject       *project,
+			    GladeWidgetInfo    *info,
+			    gboolean            free_value)
+{
+	GValue    *gvalue = NULL;
+	gint       i;
+	gchar     *id;
+
+	for (i = 0; i < info->n_properties; ++i)
+	{
+		GladePropInfo *pinfo = info->properties + i;
+		
+		id = glade_util_read_prop_name (pinfo->name);
+		
+		if (!strcmp (id, pclass->id))
+		{
+			if (property && glade_property_class_is_object (pclass))
+			{
+				/* we must synchronize this directly after loading this project
+				 * (i.e. lookup the actual objects after they've been parsed and
+				 * are present).
+				 */
+				g_object_set_data_full (G_OBJECT (property), 
+							"glade-loaded-object", 
+							g_strdup (pinfo->value), g_free);
+			}
+			else
+			{
+				gvalue = glade_property_class_make_gvalue_from_string
+					(pclass, pinfo->value, project);
+
+				if (property)
+					GLADE_PROPERTY_GET_KLASS
+						(property)->set_value (property, gvalue);
+
+				if (free_value)
+				{
+					g_value_unset (gvalue);
+					g_free (gvalue);
+				}
+			}
+			
+			if (property)
+			{
+				glade_property_i18n_set_translatable
+					(property, pinfo->translatable);
+				glade_property_i18n_set_has_context
+					(property, pinfo->has_context);
+				glade_property_i18n_set_comment
+					(property, pinfo->comment);
+
+				property->enabled = TRUE;
+			}
+
+			g_free (id);
+			break;
+		}
+		g_free (id);
+	}
+	return gvalue;
+}
+
+static GValue *
+glade_property_read_atk_prop (GladeProperty      *property, 
+			      GladePropertyClass *pclass,
+			      GladeProject       *project,
+			      GladeWidgetInfo    *info,
+			      gboolean            free_value)
+{
+	GValue    *gvalue = NULL;
+	gint       i;
+	gchar     *id;
+
+	for (i = 0; i < info->n_atk_props; ++i)
+	{
+		GladePropInfo *pinfo = info->atk_props + i;
+		
+		id = glade_util_read_prop_name (pinfo->name);
+		
+		if (!strcmp (id, pclass->id))
+		{
+			gvalue = glade_property_class_make_gvalue_from_string
+				(pclass, pinfo->value, project);
+
+			if (property)
+			{
+				glade_property_i18n_set_translatable
+					(property, pinfo->translatable);
+				glade_property_i18n_set_has_context
+					(property, pinfo->has_context);
+				glade_property_i18n_set_comment
+					(property, pinfo->comment);
+
+				property->enabled = TRUE;
+
+				GLADE_PROPERTY_GET_KLASS (property)->set_value
+					(property, gvalue);
+			}
+
+			if (free_value)
+			{
+				g_value_unset (gvalue);
+				g_free (gvalue);
+			}
+			
+			g_free (id);
+			break;
+		}
+		g_free (id);
+	}
+	return gvalue;
+}
+
+static GValue *
+glade_property_read_atk_relation (GladeProperty      *property, 
+				  GladePropertyClass *pclass,
+				  GladeProject       *project,
+				  GladeWidgetInfo    *info)
+{
+	const gchar *class_id;
+	gchar       *id, *string = NULL, *tmp;
+	gint         i;
+
+	for (i = 0; i < info->n_relations; ++i)
+	{
+		GladeAtkRelationInfo *rinfo = info->relations + i;
+			
+		id       = glade_util_read_prop_name (rinfo->type);
+		class_id = glade_property_class_atk_realname (pclass->id);
+
+		if (!strcmp (id, class_id))
+		{
+			if (string == NULL)
+				string = g_strdup (rinfo->target);
+			else
+			{
+				tmp = g_strdup_printf ("%s%s%s", string, 
+						       GPC_OBJECT_DELIMITER, rinfo->target);
+				string = (g_free (string), tmp);
+			}
+		}
+		g_free (id);
+	}
+
+	/* we must synchronize this directly after loading this project
+	 * (i.e. lookup the actual objects after they've been parsed and
+	 * are present).
+	 */
+	if (property)
+	{
+		g_object_set_data_full (G_OBJECT (property), "glade-loaded-object", 
+					g_strdup (string), g_free);
+	}
+
+	return NULL;
+}
+
+static GValue *
+glade_property_read_atk_action (GladeProperty      *property, 
+				GladePropertyClass *pclass,
+				GladeProject       *project,
+				GladeWidgetInfo    *info,
+				gboolean            free_value)
+{
+	GValue      *gvalue = NULL;
+	const gchar *class_id;
+	gchar       *id;
+	gint         i;
+
+	for (i = 0; i < info->n_atk_actions; ++i)
+	{
+		GladeAtkActionInfo *ainfo = info->atk_actions + i;
+			
+		id       = glade_util_read_prop_name (ainfo->action_name);
+		class_id = glade_property_class_atk_realname (pclass->id);
+
+		if (!strcmp (id, class_id))
+		{
+			/* Need special case for NULL values here ??? */
+			
+			gvalue = glade_property_class_make_gvalue_from_string 
+				(pclass, ainfo->description, project);
+				
+			if (property)
+				GLADE_PROPERTY_GET_KLASS
+					(property)->set_value (property, gvalue);
+
+			if (free_value)
+			{
+				g_value_unset (gvalue);
+				g_free (gvalue);
+			}
+			g_free (id);
+			break;
+		}
+		g_free (id);
+	}
+	return gvalue;
+}
+
+static GValue *
+glade_property_read_accel_prop (GladeProperty      *property, 
+				GladePropertyClass *pclass,
+				GladeProject       *project,
+				GladeWidgetInfo    *info,
+				gboolean            free_value)
+{
+	GValue      *gvalue = NULL;
+	GList       *accels = NULL;
+	gint         i;
+
+	for (i = 0; i < info->n_accels; ++i)
+	{
+		GladeAccelInfo *ainfo = info->accels + i;
+		
+		GladeAccelInfo *ainfo_dup = g_new0 (GladeAccelInfo, 1);
+		
+		ainfo_dup            = g_new0 (GladeAccelInfo, 1);
+		ainfo_dup->signal    = g_strdup (ainfo->signal);
+		ainfo_dup->key       = ainfo->key;
+		ainfo_dup->modifiers = ainfo->modifiers;
+
+		accels = g_list_prepend (accels, ainfo_dup);
+	}
+
+	gvalue = g_new0 (GValue, 1);
+	g_value_init (gvalue, GLADE_TYPE_ACCEL_GLIST);
+	g_value_take_boxed (gvalue, accels);
+
+	if (property)
+		GLADE_PROPERTY_GET_KLASS
+			(property)->set_value (property, gvalue);
+	
+	if (free_value)
+	{
+		g_value_unset (gvalue);
+		g_free (gvalue);
+	}
+
+	return gvalue;
+}
+
+
 /*******************************************************************************
                                      API
  *******************************************************************************/
 /**
  * glade_property_new:
- * @class: A #GladeWidgetClass
- * @widget: A #GladeWidget
- * @value: A #GValue
+ * @class: A #GladePropertyClass defining this property
+ * @widget: The #GladeWidget this property is created for
+ * @value: The initial #GValue of the property or %NULL
+ *         (the #GladeProperty will assume ownership of @value)
+ * @catalog_default: if specified; use any default value supplied
+ *                   by the catalog; otherwise use the introspected default.
  *
  *
- * Returns: A newly created #GladeProperty based on the
- *          given criteria
+ * Creates a #GladeProperty of type @class for @widget with @value; if
+ * @value is %NULL, then the introspected default value for that property
+ * will be used; unless otherwise specified by @catalog_default.
+ *
+ * Note that we want to use catalog defaults when creating properties for
+ * any newly created #GladeWidget; but we want to stay with the introspected
+ * defaults at load time (since the absence of the property who's default
+ * has been overridden; is interpreted as explicitly set to the default
+ * by the user).
+ *
+ * Returns: The newly created #GladeProperty
  */
 GladeProperty *
 glade_property_new (GladePropertyClass *class, 
 		    GladeWidget        *widget, 
-		    GValue             *value)
+		    GValue             *value,
+		    gboolean            catalog_default)
 {
 	GladeProperty *property;
-
+	GValue        *def;
+	
 	g_return_val_if_fail (GLADE_IS_PROPERTY_CLASS (class), NULL);
 
 	property            = 
@@ -601,22 +1042,17 @@ glade_property_new (GladePropertyClass *class,
 	property->widget    = widget;
 	property->value     = value;
 	
-	if (G_IS_PARAM_SPEC_DOUBLE  (class->pspec) ||
-	    G_IS_PARAM_SPEC_FLOAT (class->pspec)   ||
-	    G_IS_PARAM_SPEC_LONG (class->pspec)    ||
-	    G_IS_PARAM_SPEC_ULONG (class->pspec)   ||
-	    G_IS_PARAM_SPEC_INT64 (class->pspec)   ||
-	    G_IS_PARAM_SPEC_UINT64 (class->pspec)  ||
-	    G_IS_PARAM_SPEC_INT (class->pspec)     ||
-	    G_IS_PARAM_SPEC_UINT (class->pspec))
+	if (class->optional)
 		property->enabled = class->optional_default;
 	
 	if (property->value == NULL)
 	{
-		g_assert (class->def);
+		def = catalog_default ? class->def : class->orig_def;
+		g_assert (def);
+		
 		property->value = g_new0 (GValue, 1);
-		g_value_init (property->value, class->def->g_type);
-		g_value_copy (class->def, property->value);
+		g_value_init (property->value, def->g_type);
+		g_value_copy (def, property->value);
 	}
 	return property;
 }
@@ -670,7 +1106,7 @@ glade_property_default (GladeProperty *property)
  */
 gboolean
 glade_property_equals_value (GladeProperty      *property, 
-			     GValue             *value)
+			     const GValue       *value)
 {
 	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
 	return GLADE_PROPERTY_GET_KLASS (property)->equals_value (property, value);
@@ -740,7 +1176,7 @@ glade_property_set_value (GladeProperty *property, const GValue *value)
 /**
  * glade_property_set_va_list:
  * @property: a #GladeProperty
- * @va_list: a va_list with value to set
+ * @vl: a va_list with value to set
  *
  * Sets the property's value
  */
@@ -854,6 +1290,84 @@ glade_property_sync (GladeProperty *property)
 	GLADE_PROPERTY_GET_KLASS (property)->sync (property);
 }
 
+/**
+ * glade_property_load:
+ * @property: a #GladeProperty
+ *
+ * Loads the value of @property from the coresponding object instance
+ */
+void
+glade_property_load (GladeProperty *property)
+{
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+	GLADE_PROPERTY_GET_KLASS (property)->load (property);
+}
+
+/**
+ * glade_property_read:
+ * @property: a #GladeProperty or #NULL
+ * @pclass: the #GladePropertyClass
+ * @project: the #GladeProject
+ * @info: a #GladeWidgetInfo struct or a #GladeChildInfo struct if
+ *        a packing property is passed.
+ * @free_value: Whether the return value should be freed after applying
+ *              it to the property or if it should be returned in tact.
+ *
+ * Read the value and any attributes for @property from @info, assumes
+ * @property is being loaded for @project
+ *
+ * Returns: The newly created #GValue if successfull (and if @free_value == FALSE)
+ *
+ * Note that object values will only be resolved after the project is
+ * completely loaded
+ */
+GValue *
+glade_property_read (GladeProperty      *property, 
+		     GladePropertyClass *pclass,
+		     GladeProject       *project,
+		     gpointer            info,
+		     gboolean            free_value)
+{
+	GValue *ret = NULL;
+
+	g_return_val_if_fail (pclass != NULL, FALSE);
+	g_return_val_if_fail (info != NULL, FALSE);
+
+	if (pclass->packing)
+	{
+		ret = glade_property_read_packing 
+			(property, pclass, project, (GladeChildInfo *)info, free_value);
+	}
+	else switch (pclass->type)
+	{
+	case GPC_NORMAL:
+		ret = glade_property_read_normal
+			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
+		break;
+	case GPC_ATK_PROPERTY:
+		ret = glade_property_read_atk_prop
+			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
+		break;
+	case GPC_ATK_RELATION:
+		ret = glade_property_read_atk_relation
+			(property, pclass, project, (GladeWidgetInfo *)info);
+		break;
+	case GPC_ATK_ACTION:
+		ret = glade_property_read_atk_action
+			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
+		break;
+	case GPC_ACCEL_PROPERTY:
+		ret = glade_property_read_accel_prop
+			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
+		break;
+	default:
+		break;
+	}
+
+	return ret;
+}
+
+
 /**
  * glade_property_write:
  * @property: a #GladeProperty
@@ -871,6 +1385,95 @@ glade_property_write (GladeProperty *property, GladeInterface *interface, GArray
 	return GLADE_PROPERTY_GET_KLASS (property)->write (property, interface, props);
 }
 
+/**
+ * glade_property_add_object:
+ * @property: a #GladeProperty
+ * @object: The #GObject to add
+ *
+ * Adds @object to the object list in @property.
+ *
+ * Note: This function expects @property to be a #GladeParamSpecObjects
+ * or #GParamSpecObject type property.
+ */
+void
+glade_property_add_object (GladeProperty  *property,
+			   GObject        *object)
+{
+	GList *list = NULL, *new_list = NULL;
+
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+	g_return_if_fail (G_IS_OBJECT (object));
+	g_return_if_fail (GLADE_IS_PARAM_SPEC_OBJECTS (property->class->pspec) ||
+			  G_IS_PARAM_SPEC_OBJECT (property->class->pspec));
+
+	if (GLADE_IS_PARAM_SPEC_OBJECTS (property->class->pspec))
+	{
+		glade_property_get (property, &list);
+		new_list = g_list_copy (list);
+
+		new_list = g_list_append (new_list, object);
+		glade_property_set (property, new_list);
+
+		/* ownership of the list is not passed 
+		 * through glade_property_set() 
+		 */
+		g_list_free (new_list);
+	}
+	else
+	{
+		glade_property_set (property, object);
+	}
+}
+
+/**
+ * glade_property_remove_object:
+ * @property: a #GladeProperty
+ * @object: The #GObject to add
+ *
+ * Removes @object from the object list in @property.
+ *
+ * Note: This function expects @property to be a #GladeParamSpecObjects
+ * or #GParamSpecObject type property.
+ */
+void
+glade_property_remove_object (GladeProperty  *property,
+			      GObject        *object)
+{
+	GList *list = NULL, *new_list = NULL;
+
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+	g_return_if_fail (G_IS_OBJECT (object));
+	g_return_if_fail (GLADE_IS_PARAM_SPEC_OBJECTS (property->class->pspec) ||
+			  G_IS_PARAM_SPEC_OBJECT (property->class->pspec));
+
+	if (GLADE_IS_PARAM_SPEC_OBJECTS (property->class->pspec))
+	{
+		/* If object isnt in list; list should stay in tact.
+		 * not bothering to check for now.
+		 */
+		glade_property_get (property, &list);
+		new_list = g_list_copy (list);
+
+		new_list = g_list_remove (new_list, object);
+		glade_property_set (property, new_list);
+
+		/* ownership of the list is not passed 
+		 * through glade_property_set() 
+		 */
+		g_list_free (new_list);
+	}
+	else
+	{
+		glade_property_set (property, object);
+	}
+
+	glade_property_class_get_from_gvalue (property->class,
+					      property->value,
+					      &list);
+
+	glade_property_set (property, list);
+}
+
 
 /**
  * glade_property_get_tooltip:
@@ -991,3 +1594,27 @@ glade_property_get_enabled (GladeProperty *property)
 	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
 	return property->enabled;
 }
+
+
+static gint glade_property_su_stack = 0;
+
+void
+glade_property_push_superuser (void)
+{
+	glade_property_su_stack++;
+}
+
+void
+glade_property_pop_superuser (void)
+{
+	if (--glade_property_su_stack < 0)
+	{
+		g_critical ("Bug: property super user stack is corrupt.\n");
+	}
+}
+
+gboolean
+glade_property_superuser (void)
+{
+	return glade_property_su_stack > 0;
+}
