@@ -42,14 +42,19 @@
 #include "glade-property.h"
 #include "glade-property-class.h"
 #include "glade-clipboard.h"
+#include "glade-fixed.h"
 
 #define GLADE_UTIL_SELECTION_NODE_SIZE 7
 #define GLADE_UTIL_COPY_BUFFSIZE       1024
 
+#define GLADE_DEVHELP_ICON_NAME           "devhelp"
+#define GLADE_DEVHELP_FALLBACK_ICON_FILE  "devhelp.png"
+
 /* List of widgets that have selection
  */
 static GList *glade_util_selection = NULL;
 
+static GtkTooltips *glade_tooltips = NULL;
 
 /**
  * glade_util_widget_set_tooltip:
@@ -61,15 +66,19 @@ static GList *glade_util_selection = NULL;
 void
 glade_util_widget_set_tooltip (GtkWidget *widget, const gchar *str)
 {
-	GtkTooltips *tooltips;
+	g_return_if_fail (widget != NULL);
+	if (str == NULL)
+		return;
 
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (tooltips));
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (widget),
-				"tooltips", tooltips,
-				(GDestroyNotify) g_object_unref);
-	gtk_tooltips_set_tip (tooltips, widget, str, NULL);
+	if (glade_tooltips == NULL)
+	{
+		glade_tooltips = gtk_tooltips_new ();
+	
+		g_object_ref (G_OBJECT (glade_tooltips));
+		gtk_object_sink (GTK_OBJECT (glade_tooltips));	
+	}
+
+	gtk_tooltips_set_tip (glade_tooltips, widget, str, NULL);
 }
 
 /**
@@ -158,11 +167,10 @@ glade_util_get_type_from_name (const gchar *name)
 
 /**
  * glade_utils_get_pspec_from_funcname:
- * @name:
+ * @funcname: the symbol name of a function to generate a #GParamSpec
  *
- * TODO: write me
- *
- * Returns:
+ * Returns: A #GParamSpec created by the delagate function
+ *          specified by @funcname
  */
 GParamSpec *
 glade_utils_get_pspec_from_funcname (const gchar *funcname)
@@ -494,7 +502,7 @@ glade_util_hide_window (GtkWindow *window)
  * @parent: the parent #GtkWindow for the dialog
  * @action: a #GladeUtilFileDialogType to say if the dialog will open or save
  *
- * Returns: a file chooser dialog. The caller is responsible 
+ * Returns: a "glade file" file chooser dialog. The caller is responsible 
  *          for showing the dialog
  */
 GtkWidget *
@@ -502,17 +510,34 @@ glade_util_file_dialog_new (const gchar *title, GtkWindow *parent,
 			     GladeUtilFileDialogType action)
 {
 	GtkWidget *file_dialog;
+	GtkFileFilter *file_filter;
 
 	g_return_val_if_fail ((action == GLADE_FILE_DIALOG_ACTION_OPEN ||
 			       action == GLADE_FILE_DIALOG_ACTION_SAVE), NULL);
-
+	
 	file_dialog = gtk_file_chooser_dialog_new (title, parent, action,
-						    GTK_STOCK_CANCEL,
-						    GTK_RESPONSE_CANCEL,
-						    action == GLADE_FILE_DIALOG_ACTION_OPEN ?
-						    GTK_STOCK_OPEN : GTK_STOCK_SAVE,
-						    GTK_RESPONSE_OK,
-						    NULL);
+						   GTK_STOCK_CANCEL,
+						   GTK_RESPONSE_CANCEL,
+						   action == GLADE_FILE_DIALOG_ACTION_OPEN ?
+						   GTK_STOCK_OPEN : GTK_STOCK_SAVE,
+						   GTK_RESPONSE_OK,
+						   NULL);
+	
+	file_filter = gtk_file_filter_new ();
+	gtk_file_filter_add_pattern (file_filter, "*");
+	gtk_file_filter_set_name (file_filter, _("All Files"));
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_dialog), file_filter);
+
+	file_filter = gtk_file_filter_new ();
+	gtk_file_filter_add_pattern (file_filter, "*.glade");
+	gtk_file_filter_set_name (file_filter, _("Glade Files"));
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_dialog), file_filter);
+
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_dialog), file_filter);
+
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_dialog), TRUE);
+	gtk_dialog_set_default_response (GTK_DIALOG (file_dialog), GTK_RESPONSE_OK);
+	
 	gtk_window_set_position (GTK_WINDOW (file_dialog), GTK_WIN_POS_CENTER);
 
 	return file_dialog;
@@ -540,6 +565,37 @@ glade_util_replace (gchar *str, gchar a, gchar b)
 	}
 }
 
+/**
+ * glade_util_read_prop_name:
+ * @str: a string
+ *
+ * Return a usable version of a property identifier as found
+ * in a freshly parserd #GladeInterface
+ */
+gchar *
+glade_util_read_prop_name (const gchar *str)
+{
+	gchar *id;
+
+	g_return_val_if_fail (str != NULL, NULL);
+
+	id = g_strdup (str);
+
+	glade_util_replace (id, '_', '-');
+
+	if (strstr (id, "::"))
+	{
+		/* Extract the second half of "AtkObject::accessible_name"
+		 */
+		gchar **split = g_strsplit (id, "::", 0);
+		g_free (id);
+		id = g_strdup (split[1]);
+		g_strfreev (split);
+	}
+	return id;
+}
+
+
 /**
  * glade_util_duplicate_underscores:
  * @name: a string
@@ -549,13 +605,13 @@ glade_util_replace (gchar *str, gchar a, gchar b)
  *
  * Returns: a newly allocated string
  */
-char *
-glade_util_duplicate_underscores (const char *name)
+gchar *
+glade_util_duplicate_underscores (const gchar *name)
 {
-	const char *tmp;
-	const char *last_tmp = name;
-	char *underscored_name = g_malloc (strlen (name) * 2 + 1);
-	char *tmp_underscored = underscored_name;
+	const gchar *tmp;
+	const gchar *last_tmp = name;
+	gchar *underscored_name = g_malloc (strlen (name) * 2 + 1);
+	gchar *tmp_underscored = underscored_name;
 
 	for (tmp = last_tmp; *tmp; tmp = g_utf8_next_char (tmp))
 	{
@@ -893,22 +949,6 @@ glade_util_get_selection ()
 	return glade_util_selection;
 }
 
-/**
- * glade_util_selected_placeholder:
- *
- * Returns: The selected placeholder, if there
- * is only one selected item and that item is a placeholder.
- */
-GladePlaceholder *
-glade_util_selected_placeholder ()
-{
-	if (g_list_length (glade_util_selection) == 1 &&
-	    GLADE_IS_PLACEHOLDER (glade_util_selection->data))
-		return glade_util_selection->data;
-
-	return NULL;
-}
-
 /*
  * taken from gtk... maybe someday we can convince them to
  * expose gtk_container_get_all_children
@@ -927,6 +967,13 @@ gtk_container_children_callback (GtkWidget *widget,
  * glade_util_container_get_all_children:
  * @container: a #GtkContainer
  *
+ * Use this to itterate over all children in a GtkContainer,
+ * as it used _forall() instead of _foreach() (and the GTK+ version
+ * of this function is simply not exposed).
+ *
+ * Note that glade_widget_class_get_children() is the high-level
+ * abstraction and will usually end up calling this function.
+ *
  * Returns: a #GList giving the contents of @container
  */
 GList *
@@ -940,6 +987,7 @@ glade_util_container_get_all_children (GtkContainer *container)
 			      gtk_container_children_callback,
 			      &children);
 
+	/* Preserve the natural order by reversing the list */
 	return g_list_reverse (children);
 }
 
@@ -1035,11 +1083,10 @@ glade_util_gtkcontainer_relation (GladeWidget *parent, GladeWidget *widget)
 
 /**
  * glade_util_any_gtkcontainer_relation:
- * @widget: a GladeWidget
- * @parent: a GladeWidget
+ * @widgets: a #GList of #GladeWidgets
+ * @parent:  the parent #GladeWidget
  *
- *
- * Returns whether any of these widgets are parented by a GtkContainer
+ * Returns: whether any of these widgets are parented by a GtkContainer
  * through the GtkContainer interface.
  */
 gboolean
@@ -1094,207 +1141,10 @@ glade_util_count_placeholders (GladeWidget *parent)
 		}
 		g_list_free (children);
 	}
+
 	return placeholders;
 }
 
-/**
- * glade_util_paste_clipboard:
- * @placeholder: The #GladePlaceholder to paste the clipboard item to,
- *               if NULL; paste to the selected object in the active project.
- *
- * Paste the clipboard selection to the active project's 
- * selection (the project must have only one object selected).
- */
-void
-glade_util_paste_clipboard (GladePlaceholder *placeholder,
-			    GladeWidget      *parent)
-{
-	GladeClipboard     *clipboard = glade_default_app_get_clipboard ();
-	GList              *list;
-	GladeWidget        *widget = NULL;
-	gint                gtkcontainer_relations = 0;
-
-	g_return_if_fail (placeholder == NULL || GLADE_IS_PLACEHOLDER (placeholder));
-	g_return_if_fail (parent      == NULL || GLADE_IS_WIDGET (parent));
-      
-	if ((list = glade_default_app_get_selection ()) != NULL)
-	{
-		if (placeholder == NULL &&
-		    g_list_length (list) != 1)
-		{
-			glade_util_ui_message (glade_default_app_get_window(),
-					       GLADE_UI_WARN,
-					       _("Unable to paste to multiple widgets"));
-			return;
-		}
-	}
-	
-	if (g_list_length (clipboard->selection) == 0)
-	{
-		glade_util_ui_message (glade_default_app_get_window (), GLADE_UI_WARN,
-				    _("No widget selected on the clipboard"));
-		return;
-	}
-	
-	for (list = clipboard->selection; 
-	     list && list->data; list = list->next)
-	{
-		widget = list->data;
-
-		if (GTK_WIDGET_TOPLEVEL (widget->object) == FALSE && parent)
-		{
-			/* Ensure a paste is supported
-			 */
-			if (glade_util_widget_pastable (widget, parent) == FALSE)
-			{
-				glade_util_ui_message (glade_default_app_get_window (),
-						       GLADE_UI_ERROR, 
-						       _("Unable to paste widget %s to parent %s"),
-						       widget->name, parent->name);
-				return;
-			}
-
-			/* Count gtk container relations
-			 */
-			if (glade_util_gtkcontainer_relation (parent, widget))
-				gtkcontainer_relations++;
-		}
-	}
-
-	g_assert (widget);
-
-	/* Ensure enough placeholders are available */
-	if (GTK_WIDGET_TOPLEVEL (widget->object) == FALSE &&
-	    parent && parent->manager != NULL &&
-	    gtkcontainer_relations != 1) 
-	{
-		glade_util_ui_message (glade_default_app_get_window (), 
-				       GLADE_UI_WARN,
-				       _("Only one widget can be pasted at a "
-					 "time to this container"));
-		return;
-	}
-
-	if (parent && parent->manager == NULL &&
-	    glade_util_count_placeholders (parent) < gtkcontainer_relations)
-	{
-		glade_util_ui_message (glade_default_app_get_window (), 
-				       GLADE_UI_WARN,
-				       _("Insufficient amount of placeholders in "
-					 "target container"));
-		return;
-	}
-
-	glade_command_paste (clipboard->selection, parent, placeholder);
-}
-
-/**
- * glade_util_delete_clipboard:
- *
- * Cut the active project's selection.
- */
-void
-glade_util_delete_clipboard (void)
-{
-	GladeClipboard     *clipboard = glade_default_app_get_clipboard ();
-
-	glade_command_delete (clipboard->selection);
-}
-
-
-/**
- * glade_util_cut_selection:
- *
- * Cut the active project's selection.
- */
-void
-glade_util_cut_selection (void)
-{
-	GList              *widgets = NULL, *list;
-	GladeWidget        *widget;
-	
-	for (list = glade_default_app_get_selection ();
-	     list && list->data; list = list->next)
-	{
-		widget  = glade_widget_get_from_gobject (GTK_WIDGET (list->data));
-		widgets = g_list_prepend (widgets, widget);
-	}
-
-	if (widgets)
-	{
-		glade_command_cut (widgets);
-		g_list_free (widgets);
-	}
-	else
-	{
-		glade_util_ui_message (glade_default_app_get_window (),
-				       GLADE_UI_WARN, _("No widget selected!"));
-	}
-}
-
-/**
- * glade_util_copy_selection:
- *
- * Copy the active project's selection.
- */
-void
-glade_util_copy_selection (void)
-{
-	GList              *widgets = NULL, *list;
-	GladeWidget        *widget;
-	
-	for (list = glade_default_app_get_selection ();
-	     list && list->data; list = list->next)
-	{
-		if ((widget = 
-		     glade_widget_get_from_gobject (list->data)) != NULL)
-			widgets = g_list_prepend (widgets, widget);
-	}
-
-	if (widgets)
-	{
-		glade_command_copy (widgets);
-		g_list_free (widgets);
-	}
-	else
-	{
-		glade_util_ui_message (glade_default_app_get_window(),
-				       GLADE_UI_WARN,
-				       _("No widget selected!"));
-	}
-}
-
-/**
- * glade_util_delete_selection:
- *
- * Delete the active project's selection.
- */
-void
-glade_util_delete_selection (void)
-{
-	GList              *widgets = NULL, *list;
-	GladeWidget        *widget;
-
-	for (list = glade_default_app_get_selection ();
-	     list && list->data; list = list->next)
-	{
-		widget  = glade_widget_get_from_gobject (GTK_WIDGET (list->data));
-		widgets = g_list_prepend (widgets, widget);
-	}
-
-	if (widgets)
-	{
-		glade_command_delete (widgets);
-		g_list_free (widgets);
-	}
-	else
-	{
-		glade_util_ui_message (glade_default_app_get_window(),
-				       GLADE_UI_WARN,
-				       _("No widget selected!"));
-	}
-}
-
 static GtkTreeIter *
 glade_util_find_iter (GtkTreeModel *model,
 		      GtkTreeIter  *iter,
@@ -1389,10 +1239,10 @@ glade_util_basenames_match  (const gchar  *path1,
 
 /**
  * glade_util_purify_list:
- *
  * @list: A #GList
  *
- * Returns: A list with no duplicate data entries
+ * Returns: A newly allocated version of @list with no 
+ *          duplicate data entries
  */
 GList *
 glade_util_purify_list (GList *list)
@@ -1409,8 +1259,56 @@ glade_util_purify_list (GList *list)
 }
 
 /**
- * glade_util_canonical_path:
+ * glade_util_added_in_list:
+ * @old: the old #GList
+ * @new: the new #GList
  *
+ * Returns: A newly allocated #GList of elements that
+ *          are in @new but not in @old
+ *
+ */
+GList *
+glade_util_added_in_list (GList *old,
+			  GList *new)
+{
+	GList *added = NULL, *list;
+
+	for (list = new; list; list = list->next)
+	{
+		if (!g_list_find (old, list->data))
+			added = g_list_prepend (added, list->data);
+	} 
+
+	return g_list_reverse (added);
+}
+
+/**
+ * glade_util_removed_from_list:
+ * @old: the old #GList
+ * @new: the new #GList
+ *
+ * Returns: A newly allocated #GList of elements that
+ *          are in @old no longer in @new
+ *
+ */
+GList *
+glade_util_removed_from_list (GList *old,
+			      GList *new)
+{
+	GList *added = NULL, *list;
+
+	for (list = old; list; list = list->next)
+	{
+		if (!g_list_find (new, list->data))
+			added = g_list_prepend (added, list->data);
+	} 
+
+	return g_list_reverse (added);
+}
+
+
+/**
+ * glade_util_canonical_path:
  * @path: any path that may contain ".." or "." components
  *
  * Returns: an absolute path to the specified file or directory
@@ -1506,9 +1404,9 @@ glade_util_copy_file (const gchar  *src_path,
 	if (glade_util_canonical_match (src_path, dest_path))
 		return FALSE;
 
-	if (g_file_test (dest_path, G_FILE_TEST_IS_REGULAR) == TRUE)
+	if (g_file_test (dest_path, G_FILE_TEST_IS_REGULAR) != FALSE)
 		if (glade_util_ui_message
-		    (glade_default_app_get_window(), GLADE_UI_YES_OR_NO,
+		    (glade_app_get_window(), GLADE_UI_YES_OR_NO,
 		     _("%s exists.\nDo you want to replace it?"), dest_path) == FALSE)
 		    return FALSE;
 
@@ -1530,12 +1428,12 @@ glade_util_copy_file (const gchar  *src_path,
 					(dest, buffer + bytes_written, 
 					 bytes_read - bytes_written, 
 					 &written, &error)) != G_IO_STATUS_ERROR &&
-				       (bytes_written < bytes_read))
+				       (bytes_written + written) < bytes_read)
 					bytes_written += written;
 				
 				if (write_status == G_IO_STATUS_ERROR)
 				{
-					glade_util_ui_message (glade_default_app_get_window(),
+					glade_util_ui_message (glade_app_get_window(),
 							       GLADE_UI_ERROR,
 							       _("Error writing to %s: %s"),
 							       dest_path, error->message);
@@ -1549,7 +1447,7 @@ glade_util_copy_file (const gchar  *src_path,
 			
 			if (read_status == G_IO_STATUS_ERROR)
 			{
-				glade_util_ui_message (glade_default_app_get_window(),
+				glade_util_ui_message (glade_app_get_window(),
 						       GLADE_UI_ERROR,
 						       _("Error reading %s: %s"),
 						       src_path, error->message);
@@ -1564,7 +1462,7 @@ glade_util_copy_file (const gchar  *src_path,
 			if (g_io_channel_shutdown (dest, TRUE, &error) != G_IO_STATUS_NORMAL)
 			{
 				glade_util_ui_message
-					(glade_default_app_get_window(),
+					(glade_app_get_window(),
 					 GLADE_UI_ERROR,
 					 _("Error shutting down I/O channel %s: %s"),
 						       dest_path, error->message);
@@ -1574,7 +1472,7 @@ glade_util_copy_file (const gchar  *src_path,
 		}
 		else
 		{
-			glade_util_ui_message (glade_default_app_get_window(),
+			glade_util_ui_message (glade_app_get_window(),
 					       GLADE_UI_ERROR,
 					       _("Failed to open %s for writing: %s"), 
 					       dest_path, error->message);
@@ -1585,7 +1483,7 @@ glade_util_copy_file (const gchar  *src_path,
 
 		if (g_io_channel_shutdown (src, TRUE, &error) != G_IO_STATUS_NORMAL)
 		{
-			glade_util_ui_message (glade_default_app_get_window(),
+			glade_util_ui_message (glade_app_get_window(),
 					       GLADE_UI_ERROR,
 					       _("Error shutting down io channel %s: %s"),
 					       src_path, error->message);
@@ -1594,7 +1492,7 @@ glade_util_copy_file (const gchar  *src_path,
 	}
 	else 
 	{
-		glade_util_ui_message (glade_default_app_get_window(),
+		glade_util_ui_message (glade_app_get_window(),
 				       GLADE_UI_ERROR,
 				       _("Failed to open %s for reading: %s"), 
 				       src_path, error->message);
@@ -1602,3 +1500,304 @@ glade_util_copy_file (const gchar  *src_path,
 	}
 	return success;
 }
+
+/**
+ * glade_util_class_implements_interface:
+ * @class_type: A #GType
+ * @iface_type: A #GType
+ *
+ * Returns: whether @class_type implements the @iface_type interface
+ */
+gboolean
+glade_util_class_implements_interface (GType class_type, 
+				       GType iface_type)
+{
+	GType    *ifaces;
+	guint     n_ifaces, i;
+	gboolean  implemented = FALSE;
+
+	if ((ifaces = g_type_interfaces (class_type, &n_ifaces)) != NULL)
+	{
+		for (i = 0; i < n_ifaces; i++)
+			if (ifaces[i] == iface_type)
+			{
+				implemented = TRUE;
+				break;
+			}
+		g_free (ifaces);
+	}
+	return implemented;
+}
+
+
+/**
+ * glade_util_load_library:
+ * @library_name: name of the library
+ *
+ * Loads the named library from the Glade modules directory, or failing that
+ * from the standard platform specific directories.
+ *
+ * The @library_name should not include any platform specifix prefix or suffix,
+ * those are automatically added, if needed, by g_module_build_path()
+ *
+ * Returns: a #GModule on success, or %NULL on failure.
+ */
+GModule *
+glade_util_load_library (const gchar *library_name)
+{
+	gchar   *path;
+	GModule *module;
+
+	path = g_module_build_path (glade_modules_dir, library_name);
+
+	if ((module = g_module_open (path, G_MODULE_BIND_LAZY)) == NULL)
+	{
+		g_warning (_("Unable to open the module %s (%s)."),
+			   path, g_module_error());
+	}
+
+	g_free (path);
+
+	return module;
+}
+
+/**
+ * glade_util_file_is_writeable:
+ * @path:  the path to the file
+ *
+ * Checks whether the file at @path is writeable
+ *
+ * Returns: TRUE if file is writeable
+ */
+gboolean
+glade_util_file_is_writeable (const gchar *path)
+{
+	GIOChannel *channel;
+	g_return_val_if_fail (path != NULL, FALSE);
+
+	/* The only way to really know if the file is writable */
+	if ((channel = g_io_channel_new_file (path, "a+", NULL)) != NULL)
+	{
+		g_io_channel_unref (channel);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+/**
+ * glade_util_have_devhelp:
+ *
+ * Returns: whether the devhelp module is loaded
+ */
+gboolean
+glade_util_have_devhelp (void)
+{
+	static gint  have_devhelp = -1;
+	gchar       *ptr;
+	gint         cnt, ret, major, minor;
+	GError      *error = NULL;
+
+#define DEVHELP_OLD_MESSAGE  \
+    "The DevHelp installed on your system is too old, " \
+    "devhelp feature will be disabled."
+
+#define DEVHELP_MISSING_MESSAGE  \
+    "No DevHelp installed on your system, " \
+    "devhelp feature will be disabled."
+
+	if (have_devhelp < 0)
+	{
+		have_devhelp = 0;
+
+		if ((ptr = g_find_program_in_path ("devhelp")) != NULL)
+		{
+			g_free (ptr);
+
+
+			if (g_spawn_command_line_sync ("devhelp --version", 
+						       &ptr, NULL, &ret, &error))
+			{
+				/* If we have a successfull return code.. parse the output.
+				 */
+				if (ret == 0)
+				{
+					
+					if ((cnt = sscanf (ptr, "DevHelp %d.%d\n", 
+							   &major, &minor)) == 2)
+					{
+						/* Devhelp 0.12 required.
+						 */
+						if (major >= 0 &&
+						    minor >= 12)
+							have_devhelp = 1;
+						else	
+							g_message (DEVHELP_OLD_MESSAGE);
+				} else if (ptr != NULL || 
+						   strlen (ptr) > 0) 
+						g_warning ("devhelp had unparsable output: "
+							   "'%s' (parsed %d elements)", ptr, cnt);
+					else
+						g_message (DEVHELP_OLD_MESSAGE);
+				} else {
+					g_warning ("devhelp had bad return code: '%d'", ret);
+				}
+			} else {
+				g_warning ("Error trying to launch devhelp: %s",
+					   error->message);
+				g_error_free (error);
+			}
+		} else
+			g_message (DEVHELP_MISSING_MESSAGE);
+		
+	}
+	return have_devhelp;
+}
+
+/**
+ * glade_util_get_devhelp_icon:
+ * @size: the preferred icon size
+ *
+ * Creates an image displaying the devhelp icon.
+ *
+ * Returns: a #GtkImage
+ */
+GtkWidget*
+glade_util_get_devhelp_icon (GtkIconSize size)
+{
+	GtkIconTheme *icon_theme;
+	GdkScreen *screen;
+	GtkWidget *image;
+	gchar *path;
+
+	image = gtk_image_new ();
+	screen = gtk_widget_get_screen (GTK_WIDGET (image));
+	icon_theme = gtk_icon_theme_get_for_screen (screen);
+
+	if (gtk_icon_theme_has_icon (icon_theme, GLADE_DEVHELP_ICON_NAME))
+	{
+		gtk_image_set_from_icon_name (GTK_IMAGE (image), GLADE_DEVHELP_ICON_NAME, size);
+	}
+	else
+	{
+		path = g_build_filename (glade_pixmaps_dir, GLADE_DEVHELP_FALLBACK_ICON_FILE, NULL);
+
+		gtk_image_set_from_file (GTK_IMAGE (image), path);
+
+		g_free (path);
+	}
+
+	return image;
+}
+
+/**
+ * glade_util_search_devhep:
+ * @devhelp: the devhelp widget created by the devhelp module.
+ * @book: the devhelp book (or %NULL)
+ * @page: the page in the book (or %NULL)
+ * @search: the search string (or %NULL)
+ *
+ * Envokes devhelp with the appropriate search string
+ *
+ */
+void
+glade_util_search_devhelp (const gchar *book,
+			   const gchar *page,
+			   const gchar *search)
+{
+	GError *error = NULL;
+	gchar  *book_comm = NULL, *page_comm = NULL;
+	gchar  *string;
+
+	g_return_if_fail (glade_util_have_devhelp ());
+
+	if (book) book_comm = g_strdup_printf ("book:%s ", book);
+	if (page) page_comm = g_strdup_printf ("page:%s ", page);
+
+	string = g_strdup_printf ("devhelp -s \"%s%s%s\"", 
+				   book_comm ? book_comm : "",
+				   page_comm ? page_comm : "",
+				   search ? search : "");
+
+	if (g_spawn_command_line_async (string, &error) == FALSE)
+	{
+		g_warning ("Error envoking devhelp: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_free (string);
+	if (book_comm) g_free (book_comm);
+	if (page_comm) g_free (page_comm);
+}
+
+static GladeWidget *grabed_widget = NULL;
+void
+glade_util_set_grabed_widget (GladeWidget *gwidget)
+{
+	grabed_widget = gwidget;
+}
+
+gboolean
+glade_util_deep_fixed_event (GtkWidget   *widget,
+			     GdkEvent    *event,
+			     GladeWidget *gwidget)
+{
+	GladeWidget *event_widget, *search;
+
+	if (grabed_widget)
+		event_widget = grabed_widget;
+	else
+		event_widget = glade_widget_event_widget ();
+
+	/* Look for a child GladeFixed
+	 */
+	for (search = event_widget;
+	     search && search != gwidget && GLADE_IS_FIXED (search) == FALSE;
+	     search = search->parent);
+
+	if (search && GLADE_IS_FIXED (search) && search != gwidget)
+		return GLADE_WIDGET_GET_KLASS (search)->event (widget, event, search);
+
+	return FALSE;
+}
+
+GtkWidget *
+glade_util_get_placeholder_from_pointer (GtkContainer *container)
+{
+	GtkWidget *toplevel;
+	GtkWidget *retval = NULL, *child;
+	GList *c, *l;
+	gint x, y, x2, y2;
+
+	g_return_val_if_fail (GTK_IS_CONTAINER (container), NULL);
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (container));
+
+	gtk_widget_get_pointer (toplevel, &x, &y);
+	
+	for (c = l = glade_util_container_get_all_children (container);
+	     l;
+	     l = g_list_next (l))
+	{
+		child = l->data;
+		
+		if (GLADE_IS_PLACEHOLDER (child) &&
+		    GTK_WIDGET_MAPPED (child))
+		{
+			gtk_widget_translate_coordinates (toplevel, child,
+							  x, y, &x2, &y2);
+
+			
+			if (x2 >= 0 && y2 >= 0 &&
+			    x2 <= child->allocation.width &&
+			    y2 <= child->allocation.height)
+			{
+				retval = child;
+				break;
+			}
+		}
+	}
+	
+	g_list_free (c);
+
+	return retval;
+}
